@@ -1,4 +1,3 @@
-from collections import OrderedDict
 import os
 import subprocess
 import logging
@@ -17,8 +16,15 @@ from python_kemptech_api.exceptions import (
     VirtualServiceMissingLoadmasterInfo,
     RealServerMissingLoadmasterInfo,
     RealServerMissingVirtualServiceInfo,
-    LoadMasterParameterError
+    LoadMasterParameterError,
+    ValidationError,
 )
+from .utils import (
+    validate_port,
+    validate_ip,
+    validate_protocol,
+)
+
 
 requests.packages.urllib3.disable_warnings()
 log = logging.getLogger(__name__)
@@ -103,12 +109,7 @@ class HttpClient(object):
 
 
 def send_response(response):
-    """send the given response if ok.
 
-    :param response: the response to send
-    :type response: Response
-    :raises: KempTechApiErrorError
-    """
     if is_successful(response):
         return parse_to_dict(response)
     else:
@@ -137,7 +138,7 @@ class LoadMaster(HttpClient):
         )
 
     @property
-    def get_loadmaster_info(self):
+    def access_info(self):
         return {
             "endpoint": self.endpoint,
             "ip_address": self.ip_address,
@@ -145,7 +146,7 @@ class LoadMaster(HttpClient):
 
     def create_virtual_service(self, ip, port=80, protocol="tcp"):
         """VirtualService factory with pre-configured LoadMaster connection."""
-        vs = VirtualService(self.get_loadmaster_info, ip, port, protocol)
+        vs = VirtualService(self.access_info, ip, port, protocol)
         return vs
 
     def get_virtual_services(self):
@@ -153,7 +154,7 @@ class LoadMaster(HttpClient):
         data = get_data(response)
         virtual_services = []
         services = data.get('VS', [])
-        if type(services) is not list:
+        if not isinstance(services, list):
             services = [services]
         for service in services:
             virt_serv = self.build_virtual_service(service)
@@ -162,7 +163,11 @@ class LoadMaster(HttpClient):
 
     def get_virtual_service(self, index=None, address=None, port=None,
                             protocol=None):
+
         if index is None:
+            validate_ip(address)
+            validate_port(port)
+            validate_protocol(protocol)
             service_id = {"vs": address, "port": port, "prot": protocol}
         else:
             service_id = {"vs": index}
@@ -173,17 +178,11 @@ class LoadMaster(HttpClient):
 
     def build_virtual_service(self, service):
         """Create a VirtualService instance with standard defaults"""
-        virt_serv = VirtualService(self.get_loadmaster_info,
+        virt_serv = VirtualService(self.access_info,
                                    service.get('VSAddress', None),
                                    service.get('VSPort'),
                                    service.get('Protocol'))
-        virt_serv.status = service.get('Status', None)
-        virt_serv.index = service.get('Index', None)
-        virt_serv.enable = service.get('Enable', None)
-        virt_serv.forcel7 = service.get('ForceL7', None)
-        virt_serv.vstype = service.get('VStype', None)
-        virt_serv.schedule = service.get('Schedule', None)
-        virt_serv.nickname = service.get('NickName', None)
+        virt_serv.populate_default_attributes(service)
         return virt_serv
 
     def set_parameter(self, parameter, value):
@@ -365,8 +364,14 @@ class VirtualService(HttpClient, KempBaseObjectModel):
 
     def __init__(self, loadmaster_info, ip, port=80, prot="tcp"):
         self.vs = ip
+        validate_ip(ip)
+
         self.port = port
+        validate_port(port)
+
         self.prot = prot
+        validate_protocol(prot)
+
         try:
             self.endpoint = loadmaster_info["endpoint"]
         except KeyError:
@@ -386,7 +391,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         return {"vs": self.vs, "port": self.port, "prot": self.prot}
 
     @property
-    def get_virtual_service_info(self):
+    def access_info(self):
         return {
             "endpoint": self.endpoint,
             "ip_address": self.ip_address,
@@ -402,7 +407,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
             response = self._get("/modvs", self.to_api_dict())
         if is_successful(response):
             vs_data = get_data(response)
-            self.populate_virtual_service_attributes(vs_data)
+            self.populate_default_attributes(vs_data)
         else:
             raise KempTechApiException(get_error_msg(response))
 
@@ -410,7 +415,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         response = self._get("/delvs", self._get_base_parameters())
         return send_response(response)
 
-    def populate_virtual_service_attributes(self, service):
+    def populate_default_attributes(self, service):
         """Populate VirtualService instance with standard defaults"""
         self.status = service.get('Status', None)
         self.index = service.get('Index', None)
@@ -419,12 +424,17 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         self.vstype = service.get('VStype', None)
         self.schedule = service.get('Schedule', None)
         self.nickname = service.get('NickName', None)
+        self.altaddress = service.get('AltAddress', None)
 
     def create_real_server(self, ip, port=80):
         """RealServer factory with pre-configured LoadMaster connection."""
-        return RealServer(self.get_virtual_service_info, ip, port)
+        return RealServer(self.access_info, ip, port)
 
     def get_real_server(self, real_server_address=None, real_server_port=None):
+
+        validate_ip(real_server_address)
+        validate_port(real_server_port)
+
         if self.index is None:
             server_id = {
                 "vs": self.vs,
@@ -433,14 +443,13 @@ class VirtualService(HttpClient, KempBaseObjectModel):
                 "rs": real_server_address,
                 "rsport": real_server_port,
             }
-            response = self._get("/showrs", server_id)
         else:
             server_id = {
                 "vs": self.index,
                 "rs": real_server_address,
                 "rsport": real_server_port,
             }
-            response = self._get("/showrs", server_id)
+        response = self._get("/showrs", server_id)
         response_data = get_data(response)
         server = response_data["Rs"]
         real_server = self.build_real_server(server)
@@ -456,14 +465,13 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         return real_servers
 
     def build_real_server(self, server):
-        real_server = RealServer(self.get_virtual_service_info,
-                                 server.get('Addr', None),
-                                 server.get('Port', None))
-        real_server.status = server.get('Status', None)
-        real_server.forward = server.get('Forward', None)
-        real_server.enable = server.get('Enable', None)
-        real_server.weight = server.get('Weight', None)
-        real_server.limit = server.get('Limit', None)
+        if "Addr" not in server:
+            raise ValidationError('"Addr" key not present {}'.format(server))
+        if "Port" not in server:
+            raise ValidationError('"Port" key not present {}'.format(server))
+        real_server = RealServer(self.access_info, server['Addr'],
+                                 server['Port'])
+        real_server.populate_default_attributes(server)
         return real_server
 
 
@@ -471,7 +479,9 @@ class RealServer(HttpClient, KempBaseObjectModel):
 
     def __init__(self, loadmaster_virt_service_info, ip, port=80):
         self.rs = ip
+        validate_ip(ip)
         self.rsport = port
+        validate_port(port)
         try:
             self.vs = loadmaster_virt_service_info["vs"]
         except KeyError:
@@ -492,8 +502,7 @@ class RealServer(HttpClient, KempBaseObjectModel):
         super(RealServer, self).__init__()
 
     def __str__(self):
-        return 'Real Server {} on Virtual Service {} {}:{}'.format(
-            self.rs, self.prot.upper(), self.vs, self.port)
+        return 'Real Server {} on {}'.format(self.rs, self.vs)
 
     def _get_base_parameters(self):
         """Returns the bare minimum VS parameters. IP, port and protocol"""
@@ -514,7 +523,7 @@ class RealServer(HttpClient, KempBaseObjectModel):
             response = self._get("/showrs", self._get_base_parameters())
             data = get_data(response)
             rs_data = data["Rs"]
-            self.populate_real_server_attributes(rs_data)
+            self.populate_default_attributes(rs_data)
         else:
             raise KempTechApiException(get_error_msg(response))
 
@@ -522,7 +531,7 @@ class RealServer(HttpClient, KempBaseObjectModel):
         response = self._get("/delrs", self._get_base_parameters())
         return send_response(response)
 
-    def populate_real_server_attributes(self, rs_data):
+    def populate_default_attributes(self, rs_data):
         self.rsindex = rs_data.get('RsIndex', None)
         self.vsindex = rs_data.get('VSIndex', None)
         self.status = rs_data.get('Status', None)
