@@ -1,16 +1,22 @@
-from nose.tools import (assert_equal, assert_is_instance,
-                        assert_raises, assert_false,
-                        assert_in)
+from nose.tools import (assert_equal, assert_is_instance, assert_raises,
+   assert_false, assert_in)
+
 try:
-    from unittest.mock import (mock_open, patch,
-                               MagicMock, sentinel)
+    import unittest.mock as mock
 except ImportError:
-    from mock import (mock_open, patch,
-                      MagicMock, sentinel)
+    import  mock
+
+mock_open = mock.mock_open
+patch = mock.patch
+MagicMock = mock.MagicMock
+sentinel = mock.sentinel
+PropertyMock = mock.PropertyMock
+
 from requests import exceptions
 
 import python_kemptech_api.client as client
 
+LoadMaster = client.LoadMaster
 
 class MyError(Exception):
     pass
@@ -122,10 +128,16 @@ class Test_LoadMaster:
 
     def test_build_virtual_server(self):
         lm = client.LoadMaster('ip','username','password')
-        service = MagicMock()
+        lm.ip_address = "1.1.1.1"
+        with patch.object(LoadMaster,'endpoint', new_callable=PropertyMock) as mock_endpoint:
+            mock_endpoint.return_value =  "https://bal:2fourall@1.1.1.1:443/access"
 
+        service= {'VSPort': 80,
+                        'Protocol': 'TCP',
+                        'VSAddress': '1.1.1.1'
+                        }
         res = lm.build_virtual_service(service)
-        assert isinstance(res, client.VirtualService)
+        assert_is_instance(res, client.VirtualService)
 
 
 class Test_KempBaseObjectModel:
